/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.web.maven;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Date;

public class MavenRepository {
  private File targetDir;
  private Date timestamp;
  private MavenFormatter formatter;

  public MavenRepository(File targetDir) {
    this(targetDir, new MavenFormatter());
  }

  protected MavenRepository(File targetDir, MavenFormatter mavenFormatter) {
    this.targetDir = targetDir;
    this.timestamp = new Date();
    this.formatter = mavenFormatter;
  }

  public void start() {
    clean();
    try {
      FileUtils.forceMkdir(targetDir);
    } catch (IOException e) {
      throw new RuntimeException("can not create the maven repository " + targetDir);
    }
  }

  public void stop() {
    clean();
  }

  protected void clean() {
    try {
      if (targetDir != null) {
        File orgDir = new File(targetDir, "/org/");
        if (orgDir.exists()) {
          FileUtils.forceDelete(orgDir);
        }
      }
    } catch (IOException e) {
      // nothing to do
    } catch (IllegalArgumentException ie) {
      // nothing to do
    }
  }

  public void deploy(final String groupId, final String artifactId, final String version, File... jars) throws IOException {
    final StringBuilder dependencies = new StringBuilder();
    int jarCounter = 0;
    for (File jar : jars) {
      String extArtifactId = artifactId + "_ext_" + (jarCounter++);
      String extGroupId = groupId + "." + artifactId;

      dependencies.append(getExtensionPomDependency(extGroupId, extArtifactId, version));
      setupDependency(jar, extGroupId, extArtifactId, version);
    }

    SearchAndReplace replacer = new SearchAndReplace() {
      public String replace(String toSearch) {
        toSearch = toSearch.replace("$dependencies", dependencies.toString());
        toSearch = toSearch.replace("$groupId", groupId);
        toSearch = toSearch.replace("$artifactId", artifactId);
        toSearch = toSearch.replace("$metadataVersion", version);
        return toSearch.replace("$version", formatter.getSnapshotVersion(timestamp));
      }
    };
    searchAndReplace("/org/sonar/web/maven/pom.xml", getFile(groupId, artifactId, version, artifactId + "-" + formatter.getSnapshotVersion(timestamp) + ".pom"), replacer);
    copyMavenMetaData("/org/sonar/web/maven/maven-metadata.xml", getFile(groupId, artifactId, version, "maven-metadata.xml"), replacer);

  }

  private File getGroupIdDirectory(String groupId) throws IOException {
    File dir = new File(targetDir, groupId.replace('.', '/'));
    FileUtils.forceMkdir(dir);
    return dir;
  }

  private File getArtifactIdDirectory(String groupId, String artifactId) throws IOException {
    File dir = new File(getGroupIdDirectory(groupId), artifactId);
    FileUtils.forceMkdir(dir);
    return dir;
  }

  private File getVersionDirectory(String groupId, String artifactId, String version) throws IOException {
    File dir = new File(getArtifactIdDirectory(groupId, artifactId), version);
    FileUtils.forceMkdir(dir);
    return dir;
  }

  private File getFile(String groupId, String artifactId, String version, String name) throws IOException {
    return new File(getVersionDirectory(groupId, artifactId, version), name);
  }


  private StringBuilder getExtensionPomDependency(String groupId, String artifactId, String version) {
    StringBuilder dependency = new StringBuilder();
    dependency.append("\t<dependency>\n")
        .append("\t\t<groupId>")
        .append(groupId).append("</groupId>\n")
        .append("\t\t<artifactId>")
        .append(artifactId)
        .append("</artifactId>\n")
        .append("\t\t<version>")
        .append(version)
        .append("</version>\n")
        .append("\t</dependency>\n");
    return dependency;
  }

  private void setupDependency(File jar, final String groupId, final String artifactId, final String version) throws IOException {
    SearchAndReplace replacer = new SearchAndReplace() {
      public String replace(String toSearch) {
        toSearch = toSearch.replace("$artifactId", artifactId);
        toSearch = toSearch.replace("$groupId", groupId);
        toSearch = toSearch.replace("$metadataVersion", version);
        return toSearch.replace("$version", formatter.getSnapshotVersion(timestamp));
      }
    };

    searchAndReplace("/org/sonar/web/maven/pom-ext.xml",
        getFile(groupId, artifactId, version, artifactId + "-" + formatter.getSnapshotVersion(timestamp) + ".pom"), replacer);
    copyMavenMetaData("/org/sonar/web/maven/maven-metadata-ext.xml",
        getFile(groupId, artifactId, version, "maven-metadata.xml"), replacer);
    copyDependencyJar(jar, groupId, artifactId, version);
  }

  protected void searchAndReplace(String template, File target, SearchAndReplace searchAndReplace) throws IOException {
    InputStream in = this.getClass().getResourceAsStream(template);
    String templateContent = IOUtils.toString(in);
    templateContent = searchAndReplace.replace(templateContent);
    FileOutputStream out = new FileOutputStream(target);
    ByteArrayInputStream bytesIn = new ByteArrayInputStream(templateContent.getBytes());
    IOUtils.copy(bytesIn, out);
    out.close();
  }

  protected void copyMavenMetaData(String template, File target, final SearchAndReplace additionalReplacer) throws IOException {
    SearchAndReplace replacer = new SearchAndReplace() {
      public String replace(String toSearch) {
        if (additionalReplacer != null) {
          toSearch = additionalReplacer.replace(toSearch);
        }
        toSearch = toSearch.replace("$timestamp", formatter.getTimestamp(timestamp));
        toSearch = toSearch.replace("$lastUpdated", formatter.getLastUpdated(timestamp));
        return toSearch.replace("$buildNumber", formatter.getBuildNumber(timestamp));
      }
    };
    searchAndReplace(template, target, replacer);
  }

  private void copyDependencyJar(File jar, String groupId, String artifactId, String version) throws IOException {
    File target = getFile(groupId, artifactId, version, artifactId + "-" + formatter.getSnapshotVersion(timestamp) + ".jar");
    if (target.exists()) {
      target.delete();
    }
    FileUtils.copyFile(jar, target);
  }

  public interface SearchAndReplace {
    public String replace(String toSearch);
  }
}